<?php
/* vim: set expandtab tabstop=4 shiftwidth=4: */
//  
//  Copyright (c) 2004-2005 Laurent Bedubourg
//  
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//  
//  Authors: Laurent Bedubourg <lbedubourg@motion-twin.com>
//  

// METAL Specification 1.0
//
//      argument ::= expression
//
// Example:
// 
//      <hr />
//      <p metal:use-macro="here/master_page/macros/copyright">
//      <hr />
//
// PHPTAL: (here not supported)
//
//      <?php echo phptal_macro( $tpl, 'master_page.html/macros/copyright'); ? >
//

/**
 * @package phptal.php.attribute.metal
 * @author Laurent Bedubourg <lbedubourg@motion-twin.com>
 */
class PHPTAL_Php_Attribute_METAL_UseMacro extends PHPTAL_Php_Attribute
{
    static $ALLOWED_ATTRIBUTES = array(
        'fill-slot'=>'http://xml.zope.org/namespaces/metal', 
        'define-macro'=>'http://xml.zope.org/namespaces/metal', 
        'define'=>'http://xml.zope.org/namespaces/tal',
    );
    
    public function start(PHPTAL_Php_CodeWriter $codewriter)
    {
        $this->pushSlots($codewriter);
        
        foreach($this->phpelement->childNodes as $child){
            $this->generateFillSlots($codewriter,$child);
        }

        $macroname = strtr($this->expression,'-','_');

        // local macro (no filename specified) and non dynamic macro name
        // can be called directly if it's a known function (just generated or seen in previous compilation)
        if (preg_match('/^[a-z0-9_]+$/i', $macroname) && $codewriter->functionExists($macroname)) 
        {
            $code = $codewriter->getFunctionPrefix() . $macroname . '($_thistpl, $tpl)';
            $codewriter->pushCode($code);
        }
        // external macro or ${macroname}, use PHPTAL at runtime to resolve it
        else 
        {
            $code = $codewriter->interpolateTalesVarsInString($this->expression);
            $codewriter->pushCode('$tpl->_executeMacroOfTempalte('.$code.', $_thistpl)');
        }

        $this->popSlots($codewriter);
    }
    
    public function end(PHPTAL_Php_CodeWriter $codewriter)
    {
    }

    private function pushSlots(PHPTAL_Php_CodeWriter $codewriter)
    {
        // reset template slots on each macro call ?
        // 
        // NOTE: defining a macro and using another macro on the same tag 
        // means inheriting from the used macro, thus slots are shared, it 
        // is a little tricky to understand but very natural to use.
        //
        // For example, we may have a main design.html containing our main 
        // website presentation with some slots (menu, content, etc...) then
        // we may define a member.html macro which use the design.html macro
        // for the general layout, fill the menu slot and let caller templates
        // fill the parent content slot without interfering. 
        if (!$this->phpelement->hasAttributeNS('http://xml.zope.org/namespaces/metal','define-macro')){
            $codewriter->pushCode('$ctx->pushSlots()');
        }
    }

    private function popSlots(PHPTAL_Php_CodeWriter $codewriter)
    {
        // restore slots if not inherited macro
        if (!$this->phpelement->hasAttributeNS('http://xml.zope.org/namespaces/metal','define-macro')){
            $codewriter->pushCode('$ctx->popSlots()');
        }
    }
    
    private function generateFillSlots(PHPTAL_Php_CodeWriter $codewriter, PHPTAL_DOMNode $phpelement)
    {
        if (false == ($phpelement instanceOf PHPTAL_DOMElement)) 
            return;

        // if the tag contains one of the allowed attribute, we generate it
        foreach(self::$ALLOWED_ATTRIBUTES as $qname => $uri){
            if ($phpelement->hasAttributeNS($uri,$qname)){
                $phpelement->generate($codewriter);
                return;
            }
        }
        
        // recurse
        foreach($phpelement->childNodes as $child){
            $this->generateFillSlots($codewriter,$child);
        }
    }
}

?>
